/*
 *	Tuto0 for Windows: libgpu sample program
 *			 : draw sprites
 *
 *	Copyright (C) 1993-1996 by Sony Computer Entertainment Inc.
 *			All Rights Reserved
 *
 *		Display 2 moving sprites with sprite animation.
 *
 */

#if  !(defined(M_I86CM)||defined(M_I86LM)||defined(M_I86LH))
#error Must compile as Large Model.
#endif

#include <windows.h>
#include <libetc.h>
#include <string.h>

static const char instruction[4][19] = { "E,S,C,F: move MAT0", "I,J,M,L: move MAT1", "R,U: zoom", "G: quit" };

long FAR PASCAL _export WinProc (HWND hwnd, UINT message, UINT wParam, LONG lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;
	int i;
	int x, y;
	static int cw, ch;
	TEXTMETRIC tm;
	
	switch (message) {
	case WM_CREATE:
		hdc = GetDC(hwnd);
		GetTextMetrics(hdc, &tm);
		cw = tm.tmAveCharWidth;
		ch = tm.tmHeight+tm.tmExternalLeading;
		ReleaseDC(hwnd, hdc);
		break;
	case WM_PAINT:
		hdc = BeginPaint(hwnd, &ps);
		EndPaint(hwnd, &ps);
		GetClientRect(hwnd, &rect);
		x = (rect.right-cw*18)/2;
		y = (rect.bottom-ch*4)/2;
		for (i = 0; i < 4; i++, y+=ch)
			TextOut(hdc, x, y, instruction[i], strlen(instruction[i]));
		EndPaint(hwnd, &ps);
		break;
	default:
		return PadReadProc(hwnd, message, wParam, lParam);
	}
	return 0;
}

#include <libgte.h>
#include <libgpu.h>

#define DT	4	/* sprite moving unit: XvCg̈ړP */

static void load_texture4(u_short tpage[4], u_short clut[4]);

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
{
	static char szAppName[] = "WinTuto0" ;
	HWND        hwnd ;
	MSG         msg ;
	WNDCLASS    wndclass ;

	/* drawing environment and display environment.
	 * Two pair of environments are needed for double buffer:
	 * _uobt@gp̂߁AꂼQgp
	 */
	DRAWENV		draw[2];	/* drawing environment: ` */
	DISPENV		disp[2];	/* display environment: \ */
	SVECTOR		x0, x1;		/* diplay position (vs vy is used)
					   : \ʒuivx,vy ôݎgpj*/
	POLY_FT4	mat0, mat1;	/* texture mapped polygon:
					   : eNX`}bv|S */
	u_short		tpage[4];	/* texture page ID (4pages)
					   : eNX`y[W ID iSj*/
	u_short		clut[4];	/* texture CLUT ID
					   : eNX` CLUT ID SGgj*/
	int		w = 64, h = 64;	/* scale: gEk */
	int		u, v;		/* work */
	int		frame = 0;	/* counter: JE^ */
	int		id;

	if ((id = GetPortId()) == 0) {
		id = 0x1340;		/* set default port id */
	}
	
	SetPortId(id);

	if (!hPrevInstance) {
		wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
		wndclass.lpfnWndProc   = WinProc ;
		wndclass.cbClsExtra    = 0 ;
		wndclass.cbWndExtra    = 0 ;
		wndclass.hInstance     = hInstance ;
		wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
		wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
		wndclass.hbrBackground = GetStockObject (WHITE_BRUSH) ;
		wndclass.lpszMenuName  = NULL ;
		wndclass.lpszClassName = szAppName ;

		RegisterClass (&wndclass) ;
	}

	hwnd = CreateWindow (szAppName,
		"libgpu sample program for Windows",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		NULL,
		NULL,
		hInstance,
		NULL);

	ShowWindow (hwnd, nCmdShow) ;
	UpdateWindow (hwnd) ;

	/* reset keys: L[̃Zbg */
	PadInit(1);	/* argument=1 initializes keys */
	
	/* reset graphics subsystem : `E\̃Zbg */
	ResetGraph(0);		
	
	/* initialize debug print message function.
	 * FntLoad(x,y) loads font pattern to (x,y) on the frame buffer.
	 * SetDumpFnt() defines standard on-screen print stream.
	 : fobOvg֐̏
	 * FntLoad(x,y) ̓t[obt@ (x,y) ɃtHg[h
	 * SetDumpFnt() ͕WXg[`
	 */
	FntLoad(960, 256);	
	SetDumpFnt(FntOpen(16, 16, 256, 64, 0, 512));
	   
	/* set 2 drawing and display environments to construct frame
	 * double buffer.
	 * When buffer #1 is drawing, buffer #2 is displayed.
	 * When buffer #2 is drawing, buffer #1 is displayed.
	 : t[obt@Ń_uobt@\邽߂̕`E
	 * \\̂̃oݒ肷B
	 * obt@ #1 ɕ`悵ĂԂ obt@ #2 \
	 * obt@ #2 ɕ`悵ĂԂ obt@ #1 \B
	 */
	/*	buffer #1	(0,  0)-(320,240)
	 *	buffer #2	(0,240)-(320,480)
	 */
	SetDefDrawEnv(&draw[0], 0,   0, 320, 240);
	SetDefDispEnv(&disp[0], 0, 240, 320, 240);
	SetDefDrawEnv(&draw[1], 0, 240, 320, 240);
	SetDefDispEnv(&disp[1], 0,   0, 320, 240);

	/* indicate to clear BG: wi((R,G,B)=(0,0,0))ŃNA */
	draw[0].isbg = draw[1].isbg = 1;
	setRGB0(&draw[0], 0, 0, 0);	/* (R,G,B)=(0,0,0) */
	setRGB0(&draw[1], 0, 0, 0);	/* (R,G,B)=(0,0,0) */

	/* initialize polygon primiteivs. 
	 * SetPolyFT4() initialize POLY_FT4 primitive.
	 * SetShadeTex() indicates to inhibit ShadeTex option.
	 : |Sv~eBuigEkXvCgj
	 * SetPolyFT4()  POLY_FT4 v~eBu
	 * SetShadeTex() ̓v~eBu ShadeTex @\ieNX`ɃVF
	 * [fBO@\j֎~
	 */
	SetPolyFT4(&mat0);		
	SetShadeTex(&mat0, 1);		/* ShadeTex disable */
	SetPolyFT4(&mat1);
	SetShadeTex(&mat1, 1);		/* ShadeTex disable */

	/* set the initial position of each sprite
	   : XvCg̕\ʒȕl */
	x0.vx = 80-w/2;  x1.vx = 240-w/2;
	x0.vy = 120-h/2; x1.vy = 120-h/2;

	/* load the texture pattern to frame buffer, and set default tpage
	 * and CLUT  
	 : eNX`p^[t[obt@ɓ]
	 * ʂ̃eNX`y[WEeNX` CLUT  tpage, clut ɕԂB
	 */
	load_texture4(tpage, clut);

	/* display enable: \Jn */
	SetDispMask(1);

	for (;;) {
		u_long padd;

		while (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == WM_QUIT)
				return 0;
			TranslateMessage (&msg) ;
			DispatchMessage (&msg) ;
		}
		/* Idle */
		/* swap frame double buffer : _uobt@Xbv */
		PutDrawEnv(frame%2? &draw[0]:&draw[1]);
		PutDispEnv(frame%2? &disp[0]:&disp[1]);

		/* get current controller status: Rg[pbh */
		padd = PadRead(1);

		/* calculate sprite position: XvCg̈ړʂ */
		if (padd & PADLup)	x0.vy -= DT;
		if (padd & PADLdown)	x0.vy += DT;
		if (padd & PADLleft) 	x0.vx -= DT;
		if (padd & PADLright)	x0.vx += DT;

		if (padd & PADRup)	x1.vy -= DT;
		if (padd & PADRdown)	x1.vy += DT;
		if (padd & PADRleft) 	x1.vx -= DT;
		if (padd & PADRright)	x1.vx += DT;

		/* scale: gEk */
		if (padd & PADL1)	w--, h--;
		if (padd & PADR1) 	w++, h++;

		/* : Aj[V̏I */
		if (padd & PADselect) 	return 0;

		/* draw the sprite (Matrchang).
		 * Texture patterns are changed by each frame to do the  
		 * sprite animations
		 : XvCg (Matchang) `悷B
		 * XvCg (Matchang) `悷B
		 * XvCgAj[V邽߂ɁAS̃eNX`
		 * y[WɂꂽAj[Vp^[ԂɐؑւĂB
		 */
 
		/* set texture pages: eNX`y[Wݒ肷B*/
		mat0.tpage = mat1.tpage = tpage[(frame/16)%4];
		mat0.clut  = mat1.clut  = clut[(frame/16)%4];

		/* set texture point (u,v): eNX`W (u,v) ݒ */
		u = (frame%4)*64;
		v = ((frame/4)%4)*64;
		setUVWH(&mat0, u, v, 63, 63);
		setUVWH(&mat1, u, v, 63, 63);

		/* calculate sprite position: XvCg̍Wݒ */
		setXYWH(&mat0, x0.vx, x0.vy, w, h);
		setXYWH(&mat1, x1.vx, x1.vy, w, h);

		/* draw primitives: XvCgv~eBu̕` */
		DrawPrim(&mat0);
		DrawPrim(&mat1);

		/* print debug message: fobObZ[Wo */
		FntPrint("tuto0: simplest sample\n");
		FntPrint("mat0=");dumpXY0(&mat0);
		FntPrint("mat1=");dumpXY0(&mat1);
		FntPrint("frame=%d\n", frame);
		/* FntFlush(-1) indicates flush the standard stream
		   : FntFlush(-1) ŕWXg[tbV */
		FntFlush(-1);
		
		/* wait for V-BLNK: V-BLNK ̂҂ */
		VSync(0);

		/* update frame counter: t[ԍAbvf[g */
		frame++;
	}
     return 0;
}

/*
 * load texture pattern to frame buffer
 : eNX`[h֐
 */

static void load_texture4(u_short tpage[4], u_short clut[4])
{
	/* mat0, mat1, mat2, mat3 are Sprite animation image patterns.
	 * Animation patterns use 4 texture pages, and each page contains
	 * 64 patterns (64x64 size)
	 * Image format here is;
	 *	0x000-0x07f	CLUT  (256x2byte entry)
	 *	0x200-		INDEX (4bit mode, size=256x256)
	 :
	 * XvCgAj[Vf[^
	 * 64x64 ̃XvCgAj[V 64 p^[A
	 * S̃eNX`y[WɓnĊi[ĂB
	 * ey[W̐擪AhX (mat0, mat1, mat2, mat3)B
	 * éA
	 *	0x000-0x07f	CLUT  (256x2byte entry)
	 *	0x200-		INDEX (4bit mode, size=256x256)
	 */
	extern	u_long	mat0[];		/* Matchang of texture page #0 */
	extern	u_long	mat1[];		/* Matchang of texture page #1 */
	extern	u_long	mat2[];		/* Matchang of texture page #2 */
	extern	u_long	mat3[];		/* Matchang of texture page #3 */

	/* load 4 texture pages.
	 * Since each texture pattern is 4bit mode, it uses 64x256 area
	 * (not 256x256) of the frame buffer. 
	 :
	 * eNX`y[W[hBiSj
	 * Srbg[ĥ߁Aۂ̃t[obt@̗̈́A
	 * 64x256 gpȂƂɒӁB
	 */
	tpage[0] = LoadTPage(mat0+0x80, 0, 0, 640,  0, 256,256);
	tpage[1] = LoadTPage(mat1+0x80, 0, 0, 640,256, 256,256);
	tpage[2] = LoadTPage(mat2+0x80, 0, 0, 704,  0, 256,256);
	tpage[3] = LoadTPage(mat3+0x80, 0, 0, 704,256, 256,256);

	/* load 4 texture CLUTS
	   : eNX` CLUT [hBiS{j */
	clut[0] = LoadClut(mat0, 0,500);
	clut[1] = LoadClut(mat1, 0,501);
	clut[2] = LoadClut(mat2, 0,502);
	clut[3] = LoadClut(mat3, 0,503);
}

/* PadReadProc is defined in libetc */
/*
long FAR PASCAL _export PadReadProc (HWND hwnd, UINT message, UINT wParam, LONG lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			padd |= PadKeyMap[wParam];
			return 0 ;
		case WM_KEYUP:
			padd &= ~PadKeyMap[wParam];
			return 0;
		case WM_DESTROY:
			PostQuitMessage (0) ;
			return 0 ;
	}
	return DefWindowProc (hwnd, message, wParam, lParam) ;
}
*/